@@ -22,33 +22,23 @@
 #ifndef XAPIAN_INCLUDED_SAFEUNISTD_H
 #define XAPIAN_INCLUDED_SAFEUNISTD_H
 
+#ifndef PACKAGE
+# error config.h must be included first in each C++ source file
+#endif
+
 #ifndef _MSC_VER
 # include <unistd.h>
 #else
 
-// sys/types.h has a typedef for off_t so make sure we've seen that before
-// we hide it behind a #define.
-# include <sys/types.h>
-
-// MSVC doesn't even HAVE unistd.h - io.h seems the nearest equivalent.
-// We also need to do some renaming of functions to get versions which
-// work on large files.
+// io.h is the nearest equivalent to unistd.h.
 # include <io.h>
 
-# ifdef lseek
-#  undef lseek
-# endif
-
-# ifdef off_t
-#  undef off_t
-# endif
-
-# define lseek(FD, OFF, WHENCE) _lseeki64(FD, OFF, WHENCE)
-# define off_t __int64
-
 // process.h is needed for getpid().
 # include <process.h>
 
+// direct.h is needed for rmdir().
+# include <direct.h>
+
 #endif
 
 // Under mingw we probably don't need to provide our own sleep().
