@@ -1,4 +1,4 @@
-/** @file index_file.cc
+/** @file
  * @brief Handle indexing a document from a file
  */
 /* Copyright 1999,2000,2001 BrightStation PLC
@@ -169,6 +169,9 @@ index_add_default_filters()
     // option suppresses exporting picture files as pictNNNN.wmf in the current
     // directory.  Note that this option was ignored in some older versions,
     // but it was fixed in unrtf 0.20.4.
+    index_command("application/rtf",
+		  Filter("unrtf --nopict --html 2>/dev/null", "text/html",
+			 false));
     index_command("text/rtf",
 		  Filter("unrtf --nopict --html 2>/dev/null", "text/html",
 			 false));
@@ -936,7 +939,8 @@ index_mimetype(const string & file, const string & urlterm, const string & url,
 	    XmlParser xmlparser;
 	    xmlparser.parse_xml(d.gzfile_to_string());
 	    dump = xmlparser.dump;
-	} else if (mimetype == "application/vnd.ms-xpsdocument") {
+	} else if (mimetype == "application/oxps" ||
+		   mimetype == "application/vnd.ms-xpsdocument") {
 	    string cmd = "unzip -p";
 	    append_filename_argument(cmd, file);
 	    cmd += " 'Documents/1/Pages/*.fpage'";
