@@ -1,6 +1,7 @@
-/* omega.cc: Main module for omega (example CGI frontend for Xapian)
- *
- * Copyright 1999,2000,2001 BrightStation PLC
+/** @file
+ * @brief Main module for omega (example CGI frontend for Xapian)
+ */
+/* Copyright 1999,2000,2001 BrightStation PLC
  * Copyright 2001 James Aylett
  * Copyright 2001,2002 Ananova Ltd
  * Copyright 2002,2003,2004,2006,2007,2008,2009,2010,2011,2014,2015,2016,2018 Olly Betts
@@ -215,27 +216,36 @@ try {
     if (fmtname.empty())
 	fmtname = default_template;
 
-    val = cgi_params.find("MORELIKE");
-    if (enquire && val != cgi_params.end()) {
-	const string & v = val->second;
-	Xapian::docid docid = atol(v.c_str());
-	if (docid == 0) {
-	    // Assume it's MORELIKE=Quid1138 and that Quid1138 is a UID
-	    // from an external source - we just find the correspond docid
-	    Xapian::PostingIterator p = db.postlist_begin(v);
-	    if (p != db.postlist_end(v)) docid = *p;
+    auto ml = cgi_params.equal_range("MORELIKE");
+    if (enquire && ml.first != ml.second) {
+	Xapian::RSet tmprset;
+	for (auto i = ml.first; i != ml.second; ++i) {
+	    const string& v = i->second;
+	    Xapian::docid docid = atol(v.c_str());
+	    if (docid == 0) {
+		// Assume it's MORELIKE=Quid1138 and that Quid1138 is a UID
+		// from an external source - we just find the correspond docid.
+		Xapian::PostingIterator p = db.postlist_begin(v);
+		if (p != db.postlist_end(v)) docid = *p;
+	    }
+	    if (docid != 0) {
+		tmprset.add_document(docid);
+	    }
 	}
 
-	if (docid != 0) {
-	    Xapian::RSet tmprset;
-	    tmprset.add_document(docid);
-
+	if (!tmprset.empty()) {
 	    OmegaExpandDecider decider(db);
 	    set_expansion_scheme(*enquire, option);
 	    Xapian::ESet eset(enquire->get_eset(40, tmprset, &decider));
 	    string morelike_query;
 	    for (auto&& term : eset) {
-		if (!morelike_query.empty()) morelike_query += ' ';
+		if (!morelike_query.empty()) {
+		    if (default_op == Xapian::Query::OP_OR) {
+			morelike_query += ' ';
+		    } else {
+			morelike_query += " OR ";
+		    }
+		}
 		morelike_query += pretty_term(term);
 	    }
 	    add_query_string(string(), morelike_query);
