@@ -1,10 +1,11 @@
-/* query.cc: query executor for omega
- *
- * Copyright 1999,2000,2001 BrightStation PLC
+/** @file
+ * @brief query executor for omega
+ */
+/* Copyright 1999,2000,2001 BrightStation PLC
  * Copyright 2001 James Aylett
  * Copyright 2001,2002 Ananova Ltd
  * Copyright 2002 Intercede 1749 Ltd
- * Copyright 2002,2003,2004,2005,2006,2007,2008,2009,2010,2011,2013,2014,2015,2016,2017,2018,2019 Olly Betts
+ * Copyright 2002,2003,2004,2005,2006,2007,2008,2009,2010,2011,2013,2014,2015,2016,2017,2018,2019,2020 Olly Betts
  * Copyright 2008 Thomas Viehmann
  *
  * This program is free software; you can redistribute it and/or
@@ -128,7 +129,7 @@ static Xapian::QueryParser qp;
 static Xapian::NumberRangeProcessor * size_rp = NULL;
 static Xapian::Stem *stemmer = NULL;
 
-static string eval_file(const string &fmtfile);
+static string eval_file(const string& fmtfile, bool* p_not_found = nullptr);
 
 static set<string> termset;
 
@@ -329,7 +330,10 @@ read_qp_flags(const string & opt_pfx, unsigned f)
 	    f |= mask;
 	}
     }
-    return f;
+    // Always enable FLAG_ACCUMULATE so that $stoplist and $unstem report
+    // values accumulated over all query strings parsed as part of a query, not
+    // just the last one parsed.
+    return f | Xapian::QueryParser::FLAG_ACCUMULATE;
 }
 
 static querytype
@@ -908,7 +912,7 @@ static int percent;
 static double weight;
 static Xapian::doccount collapsed;
 
-static string print_caption(const string &fmt, const vector<string> &param);
+static string print_caption(const string& fmt, vector<string>& param);
 
 enum tagval {
 CMD_,
@@ -940,6 +944,7 @@ CMD_filters,
 CMD_filterterms,
 CMD_find,
 CMD_fmt,
+CMD_foreach,
 CMD_freq,
 CMD_ge,
 CMD_gt,
@@ -1083,6 +1088,7 @@ T(filters,	   0, 0, N, 0), // serialisation of current filters
 T(filterterms,	   1, 1, N, 0), // list of terms with a given prefix
 T(find,		   2, 2, N, 0), // find entry in list
 T(fmt,		   0, 0, N, 0), // name of current format
+T(foreach,	   2, 2, 1, 0), // evaluate something for every entry in a list
 T(freq,		   1, 1, N, 0), // frequency of a term
 T(ge,		   2, 2, N, 0), // test >=
 T(gt,		   2, 2, N, 0), // test >
@@ -1096,7 +1102,7 @@ T(htmlstrip,	   1, 1, N, 0), // html strip tags string (s/<[^>]*>?//g)
 T(httpheader,	   2, 2, N, 0), // arbitrary HTTP header
 T(id,		   0, 0, N, 0), // docid of current doc
 T(if,		   1, 3, 1, 0), // conditional
-T(include,	   1, 1, 1, 0), // include another file
+T(include,	   1, 2, 1, 0), // include another file
 T(json,		   1, 1, N, 0), // JSON string escaping
 T(jsonarray,	   1, 2, 1, 0), // Format list as a JSON array
 T(jsonbool,	   1, 1, 1, 0), // Format list as a JSON bool
@@ -1201,8 +1207,32 @@ write_all(int fd, const char * buf, size_t count)
     return 0;
 }
 
+static string eval(const string& fmt, vector<string>& param);
+
+/** Implements $foreach{} and $map{}. */
 static string
-eval(const string &fmt, const vector<string> &param)
+foreach(const string& list,
+	const string& pat,
+	vector<string>& param,
+	char sep = '\0')
+{
+    string result;
+    string saved_arg0 = std::move(param[0]);
+    string::size_type i = 0, j;
+    while (true) {
+	j = list.find('\t', i);
+	param[0].assign(list, i, j - i);
+	result += eval(pat, param);
+	if (j == string::npos) break;
+	if (sep) result += sep;
+	i = j + 1;
+    }
+    param[0] = std::move(saved_arg0);
+    return result;
+}
+
+static string
+eval(const string& fmt, vector<string>& param)
 {
     static map<string, const struct func_attrib *> func_map;
     if (func_map.empty()) {
@@ -1584,6 +1614,11 @@ eval(const string &fmt, const vector<string> &param)
 	    case CMD_fmt:
 		value = fmtname;
 		break;
+	    case CMD_foreach:
+		if (!args[0].empty()) {
+		    value = foreach(args[0], args[1], param);
+		}
+		break;
 	    case CMD_freq: {
 		const string& term = args[0];
 		Xapian::doccount termfreq = 0;
@@ -1725,9 +1760,18 @@ eval(const string &fmt, const vector<string> &param)
 		else if (args.size() > 2)
 		    value = eval(args[2], param);
 		break;
-	    case CMD_include:
-		value = eval_file(args[0]);
+	    case CMD_include: {
+		if (args.size() == 1) {
+		    value = eval_file(args[0]);
+		} else {
+		    bool fallback = false;
+		    value = eval_file(args[0], &fallback);
+		    if (fallback) {
+			value = eval(args[1], param);
+		    }
+		}
 		break;
+	    }
 	    case CMD_json:
 		value = args[0];
 		json_escape(value);
@@ -1925,17 +1969,7 @@ eval(const string &fmt, const vector<string> &param)
 		break;
 	    case CMD_map:
 		if (!args[0].empty()) {
-		    string l = args[0], pat = args[1];
-		    vector<string> new_args(param);
-		    string::size_type i = 0, j;
-		    while (true) {
-			j = l.find('\t', i);
-			new_args[0] = l.substr(i, j - i);
-			value += eval(pat, new_args);
-			if (j == string::npos) break;
-			value += '\t';
-			i = j + 1;
-		    }
+		    value = foreach(args[0], args[1], param, '\t');
 		}
 		break;
 	    case CMD_match:
@@ -2491,25 +2525,34 @@ eval(const string &fmt, const vector<string> &param)
 }
 
 static string
-eval_file(const string &fmtfile)
+eval_file(const string& fmtfile, bool* p_not_found)
 {
-    string err;
+    // Use -1 to indicate vet_filename() failed.
+    int eno = -1;
     if (vet_filename(fmtfile)) {
 	string file = template_dir + fmtfile;
 	string fmt;
+	errno = 0;
 	if (load_file(file, fmt)) {
 	    vector<string> noargs;
 	    noargs.resize(1);
 	    return eval(fmt, noargs);
 	}
-	err = strerror(errno);
-    } else {
-	err = "name contains '..'";
+	eno = errno;
+    }
+
+    if (p_not_found) {
+	*p_not_found = true;
+	return string();
     }
 
     // FIXME: report why!
     string msg = string("Couldn't read format template '") + fmtfile + '\'';
-    if (!err.empty()) msg += " (" + err + ')';
+    if (eno) {
+	msg += " (";
+	msg += (eno < 0 ? "name contains '..'" : strerror(eno));
+	msg += ')';
+    }
     throw msg;
 }
 
@@ -2568,7 +2611,7 @@ pretty_term(string term)
 }
 
 static string
-print_caption(const string &fmt, const vector<string> &param)
+print_caption(const string& fmt, vector<string>& param)
 {
     q0 = *(mset[hit_no]);
 
